#[macro_use]
extern crate specs_derive;
#[macro_use]
extern crate specs_system_macro;

use ggez::{event::EventHandler, Context, GameError};
use specs::prelude::*;

#[derive(Component)]
pub struct Pos {
    x: usize,
    y: usize,
}

#[derive(Component)]
pub struct Renderable {
    glyph: carpet::CP437,
    color: carpet::Color,
}

system_impl! {
    Draw(
        resource mut board: carpet::Board<carpet::CP437>,
        renderable: Renderable,
        pos: Pos,
    ) {
        board.clear();
        for (p, r) in (&pos, &renderable).join() {
            board.set_with_color([p.x, p.y], r.glyph, r.color);
        }
    }
}

#[derive(Component)]
pub struct MoveLeft;

system! {
    Leftward (
        _left: MoveLeft,
        mut pos: Pos,
    ) {
        if pos.x == 0 {
            pos.x = 79;
        } else {
            pos.x -= 1;
        }
    }
}

struct State {
    world: World,
}

impl EventHandler for State {
    fn draw(&mut self, ctx: &mut Context) -> Result<(), GameError> {
        ggez::graphics::clear(ctx, ggez::graphics::BLACK);
        self.world.fetch::<carpet::Board<carpet::CP437>>().draw(ctx)?;
        ggez::graphics::present(ctx)
    }

    fn update(&mut self, _ctx: &mut Context) -> Result<(), GameError> {
        Ok(())
    }
}

fn main() -> Result<(), GameError> {
    let mut game = carpet::GameBuilder::new()
        .name("game")
        .author("me")
        .resource_path({
            let base = std::env::var("CARGO_MANIFEST_DIR").unwrap();
            let mut path = std::path::PathBuf::from(base);
            path.push("resources");
            path
        })
        .tileset("/terminal8x8.jpg", [8, 8])
        .map_size(80, 50)
        .build()?;
    game.register::<Pos>();
    game.register::<Renderable>();
    game.register::<MoveLeft>();
    game.world.print([1, 1], "Hello, world!");

    game.create_entity()
        .with(Pos { x: 40, y: 25 })
        .with(Renderable {
            glyph: carpet::CP437::from_char('A'),
            color: carpet::Color::Blue,
        })
        .build();

    for i in 0..10 {
        game.create_entity()
            .with(Pos { x: i * 7, y: 20 })
            .with(Renderable {
                glyph: carpet::CP437::from_char('X'),
                color: carpet::Color::Red,
            })
            .with(MoveLeft)
            .build();
    }
    game.run_with_systems(|world| {
        Draw.run_now(&world);
        Leftward.run_now(&world);
    })
}
// Local Variables:
// cargo-process--command-build: "build --release"
// cargo-process--command-run: "run --release --bin ch2"
// End:
